@@ -2,10 +2,9 @@
 
 /* Metacity interface for talking to GTK+ UI module */
 
-/* 
+/*
  * Copyright (C) 2002 Havoc Pennington
- * stock icon code Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -15,7 +14,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -28,8 +27,6 @@
 #include "core.h"
 #include "theme.h"
 
-#include "inlinepixbufs.h"
-
 #include <string.h>
 #include <stdlib.h>
 #include <cairo-xlib.h>
@@ -257,7 +254,7 @@ meta_ui_remove_event_func (Display       *xdisplay,
                            gpointer       data)
 {
   g_return_if_fail (ef != NULL);
-  
+
   gdk_window_remove_filter (NULL, filter_func, ef);
 
   g_free (ef);
@@ -304,14 +301,11 @@ meta_ui_free (MetaUI *ui)
 }
 
 void
-meta_ui_get_frame_geometry (MetaUI *ui,
-                            Window frame_xwindow,
-                            int *top_height, int *bottom_height,
-                            int *left_width, int *right_width)
+meta_ui_get_frame_borders (MetaUI *ui,
+                           Window frame_xwindow,
+                           MetaFrameBorders *borders)
 {
-  meta_frames_get_geometry (ui->frames, frame_xwindow,
-                            top_height, bottom_height,
-                            left_width, right_width);
+  meta_frames_get_borders (ui->frames, frame_xwindow, borders);
 }
 
 Window
@@ -330,7 +324,7 @@ meta_ui_create_frame_window (MetaUI *ui,
   gint attributes_mask;
   GdkWindow *window;
   GdkVisual *visual;
-  
+
   /* Default depth/visual handles clients with weird visuals; they can
    * always be children of the root depth/visual obviously, but
    * e.g. DRI games can't be children of a parent that has the same
@@ -373,7 +367,7 @@ meta_ui_create_frame_window (MetaUI *ui,
 		    &attrs, attributes_mask);
 
   gdk_window_resize (window, width, height);
-  
+
   meta_frames_manage_window (ui->frames, GDK_WINDOW_XID (window), window);
 
   return GDK_WINDOW_XID (window);
@@ -435,6 +429,13 @@ meta_ui_unflicker_frame_bg (MetaUI *ui,
                             target_width, target_height);
 }
 
+void
+meta_ui_update_frame_style (MetaUI *ui,
+                            Window  xwindow)
+{
+  meta_frames_update_frame_style (ui->frames, xwindow);
+}
+
 void
 meta_ui_repaint_frame (MetaUI *ui,
                        Window xwindow)
@@ -594,15 +595,17 @@ meta_ui_get_default_window_icon (MetaUI *ui)
                                                    0,
                                                    NULL);
       else
-        default_icon = gdk_pixbuf_new_from_inline (-1, window_data,
-                                                   FALSE,
+          default_icon = gtk_icon_theme_load_icon (theme,
+                                                   "gtk-missing-image",
+                                                   META_ICON_WIDTH,
+                                                   0,
                                                    NULL);
 
       g_assert (default_icon);
     }
 
   g_object_ref (G_OBJECT (default_icon));
-  
+
   return default_icon;
 }
 
@@ -627,15 +630,17 @@ meta_ui_get_default_mini_icon (MetaUI *ui)
                                                    0,
                                                    NULL);
       else
-        default_icon = gdk_pixbuf_new_from_inline (-1, mini_window_data,
-                                                   FALSE,
+          default_icon = gtk_icon_theme_load_icon (theme,
+                                                   "gtk-missing-image",
+                                                   META_MINI_ICON_WIDTH,
+                                                   0,
                                                    NULL);
 
       g_assert (default_icon);
     }
 
   g_object_ref (G_OBJECT (default_icon));
-  
+
   return default_icon;
 }
 
@@ -666,7 +671,7 @@ meta_text_property_to_utf8 (Display             *xdisplay,
   char **list;
   int count;
   char *retval;
-  
+
   list = NULL;
 
   display = gdk_x11_lookup_xdisplay (xdisplay);
@@ -684,20 +689,17 @@ meta_text_property_to_utf8 (Display             *xdisplay,
       retval = list[0];
       list[0] = g_strdup (""); /* something to free */
     }
-  
+
   g_strfreev (list);
 
   return retval;
 }
 
 void
-meta_ui_theme_get_frame_borders (MetaUI *ui,
-                                 MetaFrameType      type,
-                                 MetaFrameFlags     flags,
-                                 int               *top_height,
-                                 int               *bottom_height,
-                                 int               *left_width,
-                                 int               *right_width)
+meta_ui_theme_get_frame_borders (MetaUI           *ui,
+                                 MetaFrameType     type,
+                                 MetaFrameFlags    flags,
+                                 MetaFrameBorders *borders)
 {
   int text_height;
   PangoContext *context;
@@ -731,15 +733,14 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
 
       meta_theme_get_frame_borders (meta_theme_get_current (),
                                     type, text_height, flags,
-                                    top_height, bottom_height,
-                                    left_width, right_width);
+                                    borders);
 
       if (free_font_desc)
         pango_font_description_free (free_font_desc);
     }
   else
     {
-      *top_height = *bottom_height = *left_width = *right_width = 0;
+      meta_frame_borders_clear (borders);
     }
 
   if (style != NULL)
@@ -821,24 +822,24 @@ meta_ui_parse_accelerator (const char          *accel,
   GdkModifierType gdk_mask = 0;
   guint gdk_sym = 0;
   guint gdk_code = 0;
-  
+
   *keysym = 0;
   *keycode = 0;
   *mask = 0;
 
   if (strcmp (accel, "disabled") == 0)
     return TRUE;
-  
+
   meta_ui_accelerator_parse (accel, &gdk_sym, &gdk_code, &gdk_mask);
   if (gdk_mask == 0 && gdk_sym == 0 && gdk_code == 0)
     return FALSE;
 
   if (gdk_sym == None && gdk_code == 0)
     return FALSE;
-  
+
   if (gdk_mask & GDK_RELEASE_MASK) /* we don't allow this */
     return FALSE;
-  
+
   *keysym = gdk_sym;
   *keycode = gdk_code;
 
@@ -862,7 +863,7 @@ meta_ui_parse_accelerator (const char          *accel,
     *mask |= META_VIRTUAL_HYPER_MASK;
   if (gdk_mask & GDK_META_MASK)
     *mask |= META_VIRTUAL_META_MASK;
-  
+
   return TRUE;
 }
 
@@ -872,7 +873,7 @@ meta_ui_accelerator_name  (unsigned int        keysym,
                            MetaVirtualModifier mask)
 {
   GdkModifierType mods = 0;
-        
+
   if (keysym == 0 && mask == 0)
     {
       return g_strdup ("disabled");
@@ -910,19 +911,19 @@ meta_ui_parse_modifier (const char          *accel,
   GdkModifierType gdk_mask = 0;
   guint gdk_sym = 0;
   guint gdk_code = 0;
-  
+
   *mask = 0;
 
   if (accel == NULL || strcmp (accel, "disabled") == 0)
     return TRUE;
-  
+
   meta_ui_accelerator_parse (accel, &gdk_sym, &gdk_code, &gdk_mask);
   if (gdk_mask == 0 && gdk_sym == 0 && gdk_code == 0)
     return FALSE;
 
   if (gdk_sym != None || gdk_code != 0)
     return FALSE;
-  
+
   if (gdk_mask & GDK_RELEASE_MASK) /* we don't allow this */
     return FALSE;
 
@@ -946,7 +947,7 @@ meta_ui_parse_modifier (const char          *accel,
     *mask |= META_VIRTUAL_HYPER_MASK;
   if (gdk_mask & GDK_META_MASK)
     *mask |= META_VIRTUAL_META_MASK;
-  
+
   return TRUE;
 }
 
